@@ -19,11 +19,8 @@
 #include <stdint.h>
 #include <pthread.h>
 #include "eisl.h"
-#include "compat/nana_stubs.h"
-#include "mem.h"
-#include "fmt.h"
-#include "except.h"
-#include "str.h"
+
+
 
 void init_cell(void)
 {
@@ -31,6 +28,7 @@ void init_cell(void)
 
     /* initialize heap area */
     for (addr = 0; addr < cell_size; addr++) {
+	heap[addr].name = NULL;
 	SET_CDR(addr, addr + 1);
     }
     hp[0] = 0;
@@ -426,9 +424,10 @@ int make_sym1(const char *pname)
 
     addr = freshcell();
     SET_TAG(addr, SYM);
-    TRY heap[addr].name = Str_dup(pname, 1, 0, 1);
-    EXCEPT(Mem_Failed) error(MALLOC_OVERF, "make_sym", NIL, 0);
-    END_TRY;
+    heap[addr].name = eisl_strdup(pname);
+    if (heap[addr].name == NULL) {
+	error(MALLOC_OVERF, "make_sym", NIL, 0);
+    }
     SET_CAR(addr, NIL);
     SET_CDR(addr, NIL);
     SET_AUX(addr, csymbol);	// class symbol
@@ -537,9 +536,10 @@ int make_func(const char *pname, int addr)
 
     val = freshcell();
     SET_TAG(val, FUNC);
-    TRY heap[val].name = Str_dup(pname, 1, 0, 1);
-    EXCEPT(Mem_Failed) error(MALLOC_OVERF, "make_func", NIL, 0);
-    END_TRY;
+    heap[val].name = eisl_strdup(pname);
+    if (heap[val].name == NULL) {
+	error(MALLOC_OVERF, "make_func", NIL, 0);
+    }
     SET_CAR(val, addr);
     SET_CDR(val, ep[0]);	/* local environment */
     SET_AUX(val, cfunction);	/* class function */
@@ -578,9 +578,10 @@ int make_generic(char *pname, int lamlist, int body)
 
     val = freshcell();
     SET_TAG(val, GENERIC);
-    TRY heap[val].name = Str_dup(pname, 1, 0, 1);
-    EXCEPT(Mem_Failed) error(MALLOC_OVERF, "make_generic", NIL, 0);
-    END_TRY;
+    heap[val].name = eisl_strdup(pname);
+    if (heap[val].name == NULL) {
+	error(MALLOC_OVERF, "make_generic", NIL, 0);
+    }
     SET_CAR(val, lamlist);
     SET_OPT(val, count_args(lamlist));	/* amount of argument */
     SET_CDR(val, NIL);
@@ -684,9 +685,10 @@ int make_vec(int n, int obj)
     int res, i, *vec;
 
     res = freshcell();
-    TRY vec = (int *) ALLOC(sizeof(int) * n);
-    EXCEPT(Mem_Failed) error(MALLOC_OVERF, "make_vector", NIL, 0);
-    END_TRY;
+    vec = (int *) malloc(sizeof(int) * n);
+    if (vec == NULL) {
+	error(MALLOC_OVERF, "make_vector", NIL, 0);
+    }
     SET_TAG(res, VEC);
     SET_VEC(res, vec);
     for (i = 0; i < n; i++)
@@ -785,9 +787,10 @@ int make_arr(int ls, int obj)
 	size = 1;
 
     res = freshcell();
-    TRY vec = (int *) ALLOC(sizeof(int) * size);
-    EXCEPT(Mem_Failed) error(MALLOC_OVERF, "array", NIL, 0);
-    END_TRY;
+    vec = (int *) malloc(sizeof(int) * size);
+    if (vec == NULL) {
+	error(MALLOC_OVERF, "array", NIL, 0);
+    }
     if (nullp(ls1)) {
 	SET_TAG(res, ARR);
 	SET_CDR(res, ls1);
@@ -815,9 +818,10 @@ int make_str(const char *string)
 
     addr = freshcell();
     SET_TAG(addr, STR);
-    TRY heap[addr].name = Str_dup(string, 1, 0, 1);
-    EXCEPT(Mem_Failed) error(MALLOC_OVERF, "make_str", NIL, 0);
-    END_TRY;
+    heap[addr].name = eisl_strdup(string);
+    if (heap[addr].name == NULL) {
+	error(MALLOC_OVERF, "make_str", NIL, 0);
+    }
     SET_AUX(addr, cstring);	/* class string */
     return (addr);
 }
@@ -900,9 +904,10 @@ int make_char(const char *pname)
 
     addr = freshcell();
     SET_TAG(addr, CHR);
-    TRY heap[addr].name = (char *) ALLOC(CHARSIZE);
-    EXCEPT(Mem_Failed) error(MALLOC_OVERF, "make_char", NIL, 0);
-    END_TRY;
+    heap[addr].name = (char *) malloc(CHARSIZE);
+    if (heap[addr].name == NULL) {
+	error(MALLOC_OVERF, "make_char", NIL, 0);
+    }
     if (!isUni2(pname[0]) && !isUni3(pname[0]) && !isUni4(pname[0])
 	&& !isUni5(pname[0]) && !isUni6(pname[0])) {
 	heap[addr].name[0] = char_entity;
@@ -930,9 +935,10 @@ int make_class(const char *pname, int superclass)
 
     addr = freshcell();
     SET_TAG(addr, CLASS);
-    TRY heap[addr].name = Str_dup(pname, 1, 0, 1);
-    EXCEPT(Mem_Failed) error(MALLOC_OVERF, "make_class", NIL, 0);
-    END_TRY;
+    heap[addr].name = eisl_strdup(pname);
+    if (heap[addr].name == NULL) {
+	error(MALLOC_OVERF, "make_class", NIL, 0);
+    }
     SET_CAR(addr, superclass);
     SET_CDR(addr, NIL);
     SET_AUX(addr, NIL);
@@ -1073,3 +1079,70 @@ int make_dummy(void)
     SET_AUX(res, cnull);
     return (res);
 }
+
+
+char *eisl_strdup(const char *s)
+{
+    size_t len = strlen(s) + 1;
+    char *copy = malloc(len);
+    if (!copy)
+	return NULL;
+    memcpy(copy, s, len);
+    return copy;
+}
+
+char *eisl_strcatv(const char *first, ...)
+{
+    va_list ap;
+    size_t total_len = 0;
+
+    va_start(ap, first);
+    for (const char *s = first; s != NULL; s = va_arg(ap, const char *)) {
+	total_len += strlen(s);
+    }
+    va_end(ap);
+
+    char *result = (char *) malloc(total_len + 1);	// +1 for '\0'
+    if (!result) {
+	fprintf(stderr, "malloc failed in str_catv\n");
+	exit(1);
+    }
+
+    result[0] = '\0';
+    va_start(ap, first);
+    for (const char *s = first; s != NULL; s = va_arg(ap, const char *)) {
+	strcat(result, s);
+    }
+    va_end(ap);
+
+    return result;
+}
+
+char *eisl_strcat(const char *s1, const char *s2)
+{
+    if (!s1)
+	s1 = "";
+    if (!s2)
+	s2 = "";
+
+    size_t len1 = strlen(s1);
+    size_t len2 = strlen(s2);
+
+    char *res = (char *) malloc(len1 + len2 + 1);
+    if (!res) {
+	fprintf(stderr, "malloc failed in str_cat\n");
+	return NULL;
+    }
+    memcpy(res, s1, len1);
+    memcpy(res + len1, s2, len2);
+    res[len1 + len2] = '\0';
+
+    return res;
+}
+
+const char *eisl_strchr(const char *str, char ch)
+{
+    if (!str)
+	return NULL;
+    return strchr(str, ch);
+}
